<div class="search-bar">
  <form role="search" action="{{ pathto('search') }}" method="get">
    <input type="search" name="q" placeholder="Search" />
    <button type="submit">
      <img src="https://raw.githubusercontent.com/thoughtbot/refills/master/source/images/search-icon-black.png" alt="Search Icon">
    </button>
  </form>
</div>
