<!DOCTYPE html>
<html lang="en">
<head>

  <!-- Basic Page Needs
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta charset="utf-8">
  <title>{{ title }} &mdash; {{ shorttitle }}</title>
  <meta name="description" content="{{ DESCRIPTION }}">
  <meta name="author" content="{{ AUTHOR }}">

  <!-- Mobile Specific Metas
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta name="viewport" content="width=device-width, initial-scale=1">

  <!-- FONT
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <script src="https://ajax.googleapis.com/ajax/libs/webfont/1.6.16/webfont.js"></script>
  <script>
    WebFont.load({
      google: {
        families: ['Source Sans Pro']
      }
    });
  </script>

  <!-- CSS
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="stylesheet" href="//{{ css_server }}/theme/css/main.css">
  <link rel="stylesheet" href="{{ pathto('_static/custom.css', 1) }}">
  

  <!-- Scripts
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->

  <script src="//ajax.googleapis.com/ajax/libs/jquery/2.1.1/jquery.min.js"></script>
  <script type="text/javascript" async
  src="https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.1/MathJax.js?config=TeX-MML-AM_CHTML">
  </script>
  <script src="{{ pathto('_static/' + theme_js, 1) }}"></script>
  {% for link in js_includes %}
     <script src="{{ pathto('_static/' + link, 1) }}"></script>
  {% endfor %}

  <!-- Favicon
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="icon" type="image/png" href="{{ pathto('_static/' + theme_favicon, 1) }}">

  <!-- Canonical
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="canonical" href="http://holoviews.org/{{ pagename }}{{ file_suffix }}" />

</head>
<body class="">

  <header class="navigation">
    <div class="wrapper">
      <a href="{{ SITE_URL }}" class="logo">
        <img src="{{ pathto('_static/images/logo.png', 1) }}" alt="Bokeh Logo">
      </a>
      <a href="{{ SITE_URL }}" class="navigation-menu logo-text">HoloViews</a>
      <a href="javascript:void(0)" class="navigation-menu-button" id="js-mobile-menu">Menu</a>
      <nav>
        <ul id="js-navigation-menu" class="navigation-menu show">
          {% for name, link in NAV %}
              <li class="nav-link"><a href="{{ link }}">{{ name }}</a></li>
          {% endfor %}
        </ul>
      </nav>
    </div>
  </header>
  <div class="second-nav">
    <nav>
      <ul class="navigation-menu show">
        {%- for name, link in NAV_DOCS  %}
            <li class="nav-link doc-head"><a href="{{ pathto(link) }}">{{ name }}</a></li>
        {%- endfor %}
      </ul>
    </nav>
    <div class="navigation-tools">
      {% include 'includes/searchbox.html' %}
    </div>
  </div>

<!-- MAIN BODY OF DOCS –––––––––––––––––– -->
  <div class="docs section">
    <div class="toc">
      {{ toctree(maxdepth=4, collapse=True, includehidden=True) }}
    </div>
    <div class="content">
        {% block body %}{% endblock %}
    </div>
  </div>

<!-- END MAIN BODY OF DOCS ––––––––––––– -->

<footer class="footer">
  <div class="footer-links">
    <ul>
      <li><span class="footer-title">Links</span></li>
      {% for name, link in LINKS %}
          <li><a href="{{ link }}">{{ name }}</a></li>
      {% endfor %}
    </ul>
    <ul>
      <li><span class="footer-title">Join Us</span></li>
      {% for name, link in SOCIAL %}
          <li><a href="{{ link }}">{{ name }}</a></li>
      {% endfor %}
    </ul>
    <ul class="copyright">
      <li>{{ copyright }}</li>
    </ul>
  </div>
</footer>

<!-- Google Analytics -->
{% include 'includes/ga.html' %}

<!-- MathJax Config -->
<script type="text/x-mathjax-config">
MathJax.Hub.Config({
  tex2jax: {inlineMath: [['$','$'], ['\\(','\\)']]}
});
</script>

<!-- End Document
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
</body>

</html>
