{%- extends "layout.html" %}
{% set title = _('Search') %}
{% block body %}
  <h1 id="search-documentation">{{ _('Search') }}</h1>
  <div id="fallback" class="admonition warning">
    <script type="text/javascript">$('#fallback').hide();</script>
    <p>{% trans %}Please activate JavaScript to enable the search functionality{% endtrans %} </p>
  </div>

  <p>
    {% trans %}From here you can search these documents. Enter your search
    words into the box below and click "search". Note that the search
    function will automatically search for all of the words. Pages
    containing fewer words won't appear in the result list.{% endtrans %}
  </p>

  <form class="form-search">
    <input type="text" class="input-medium search-query" name="q" value="" />
    <input type="submit" class="btn btn-default" value="{{ _('search') }}" />
    <span id="search-progress" style="padding-left: 10px"></span>
  </form>

  <div id="search-results">
	<script>
	  (function() {
      var cx = '006807479272082416678:p6n_f0d8taw';
      var gcse = document.createElement('script');
      gcse.type = 'text/javascript';
      gcse.async = true;
      gcse.src = 'https://cse.google.com/cse.js?cx=' + cx;
      var s = document.getElementsByTagName('script')[0];
      s.parentNode.insertBefore(gcse, s);
	  })();
	</script>
    <gcse:searchresults-only></gcse:searchresults-only>

  </div>

{% endblock %}
