.wy-menu-vertical ul {
    margin: 0 0 0 0 !important;
}

div.affix {
    position: relative !important;
}

body {
    font-family: "Lato","proxima-nova","Helvetica Neue",Arial,sans-serif !important;
}

.a:hover {
    color: #b3b3b3 !important;
}

a.fa {
    font-size: 200%;
}

.code :not(h1, h2, h3, h4, h5, h6) {
    font-size: 80%;
}

div.wy-nav-content {
    max-width: 90%;
}

input {
    height: 30px !important;
}

code {
    font-size: inherit !important;
    color: #000 !important;
}

.rst-content dl:not(.docutils) dt {
    margin: 0;
    color: black;
    background: none;
    border-top: none;
}

.rst-content dd {
   line-height: 2;
}
